<?php

/**
 * @file
 * Rules integration for the Facebook Application Framework module.
 *
 * @todo
 * Implement posting events (including RSVP's) and checkins;
 *
 */

/**
 * Implements hook_rules_event_info().
 */
function fbapp_rules_rules_event_info() {
  return array(
    'fbapp_user_connected' => array(
      'label' => t('User has connected his Facebook account'),
      'module' => 'Facebook Application Framework',
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User that allowed Facebook App access')
        ),
      ),
    ),
    'fbapp_user_disconnected' => array(
      'label' => t('User has disconnected his Facebook account.'),
      'module' => 'Facebook Application Framework',
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User that revoked Facebook App access.')
        ),
      ),
    ),
    'fbapp_user_login' => array(
      'label' => t('User has logged in via Facebook'),
      'module' => 'Facebook Application Framework',
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('User who has logged in via Facebook.')
        ),
      ),
    ),
    'fbapp_user_data_modified' => array(
      'label' => t('An user has changed some of his/hers data on Facebook'),
      'module' => 'Facebook Application Framework',
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('The user whose data changed.')
        ),
        'fields' => array(
          'type' => 'string',
          'label' => t('The fields that were affected.')
        ),
        'time' => array(
          'type' => 'date',
          'label' => t('The time of the changes.')
        ),
      ),
    ),
    'fbapp_user_permissions_modified' => array(
      'label' => t('An user has changed granted permissions on Facebook'),
      'module' => 'Facebook Application Framework',
      'arguments' => array(
        'account' => array(
          'type' => 'user',
          'label' => t('The user whose data changed.')
        ),
        'permissions' => array(
          'type' => 'string',
          'label' => t('The permissions that were affected.')
        ),
        'time' => array(
          'type' => 'date',
          'label' => t('The time of the changes.')
        ),
      ),
    ),
  );
}

/**
 * Implements hook_rules_action_info().
 */
function fbapp_rules_rules_action_info() {
  return array(
    'fbapp_rules_publish_post' => array(
      'label' => t('Publish post to Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target', 'message', 'actions'),
      'new variables' => array(
        'post_id' => array(
          'type' => 'string',
          'label' => t('Created Post ID'),
        ),
      ),
    ),
    'fbapp_rules_publish_link' => array(
      'label' => t('Publish link to Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target', 'link', 'message', 'picture', 'name', 'caption', 'description'),
      'new variables' => array(
        'link_id' => array(
          'type' => 'string',
          'label' => t('Created Link ID'),
        ),
      ),
    ),
    'fbapp_rules_publish_photo' => array(
      'label' => t('Publish photo to Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target', 'source', 'message', 'no_story'),
      'new variables' => array(
        'photo_id' => array(
          'type' => 'string',
          'label' => t('Created Photo ID'),
        ),
      ),
    ),
    'fbapp_rules_comment_object' => array(
      'label' => t('Comment an object on Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target', 'message'),
      'new variables' => array(
        'comment_id' => array(
          'type' => 'string',
          'label' => t('Created Comment ID'),
        ),
      ),
    ),
    'fbapp_rules_like_object' => array(
      'label' => t('Like an object on Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target'),
    ),
    'fbapp_rules_unlike_object' => array(
      'label' => t('Unlike an object on Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target'),
    ),
    'fbapp_rules_delete_object' => array(
      'label' => t('Delete object from Facebook'),
      'module' => 'Facebook Application Framework',
      'eval input' => array('access_token', 'target'),
    ),
  );
}

/**
 * Publish post action configuration form.
 */
function fbapp_rules_publish_post_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '', 'message' => '', 'actions' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application creating the post.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target feed to post on (user\'s wall, page, app, group, etc.)'),
    '#required' => TRUE,
  );
  $form['settings']['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The post text.'),
    '#required' => TRUE,
  );
  $form['settings']['actions'] = array(
    '#type' => 'textarea',
    '#title' => t('Actions'),
    '#default_value' => $settings['actions'],
    '#description' => t('Optional. An action link to add to the post. A JSON object containing the "name" and "link".<br />Ex.: \'{"name": "View on Zombo", "link": "http://www.zombo.com"}\''),
  );
}

/**
 * Publish post to Facebook.
 */
function fbapp_rules_publish_post($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
    'message' => $settings['message'],
    'actions' => $settings['actions'],
  );
  $post = fbapp_graph_request("{$target}/feed", $parameters, 'POST');
  return array('post_id' => $post['id']);
}

/**
 * Publish post action configuration form.
 */
function fbapp_rules_publish_link_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '', 'link' => '', 'message' => '', 'picture' => '', 'name' => '', 'caption' => '', 'description' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application posting the link.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target feed to post the link on (user\'s wall, page or application.)'),
    '#required' => TRUE,
  );
  $form['settings']['link'] = array(
    '#type' => 'textfield',
    '#title' => t('Link URL'),
    '#default_value' => $settings['link'],
    '#description' => t('The Link URL.'),
    '#required' => TRUE,
  );
  $form['settings']['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The optional message from the user about this link.'),
    '#required' => FALSE,
  );
  $form['settings']['picture'] = array(
    '#type' => 'textfield',
    '#title' => t('Picture'),
    '#default_value' => $settings['picture'],
    '#description' => t('A URL to the thumbnail image used in the link post.'),
    '#required' => FALSE,
  );
  $form['settings']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Link name'),
    '#default_value' => $settings['name'],
    '#description' => t('The name of the link.'),
    '#required' => TRUE,
  );
  $form['settings']['caption'] = array(
    '#type' => 'textarea',
    '#title' => t('Caption'),
    '#default_value' => $settings['caption'],
    '#description' => t('The link caption.'),
    '#required' => FALSE,
  );
  $form['settings']['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#default_value' => $settings['description'],
    '#description' => t('A description of the link (appears beneath the link caption).'),
    '#required' => TRUE,
  );
}

/**
 * Publish link to Facebook.
 */
function fbapp_rules_publish_link($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
    'message' => $settings['message'],
    'link' => $settings['link'],
    'picture' => $settings['picture'],
    'name' => $settings['name'],
    'caption' => $settings['caption'],
    'description' => $settings['description'],
  );
  // This should be posted to /links to have a "share" link available, but doesn't work.
  $link = fbapp_graph_request("{$target}/feed", $parameters, 'POST');
  return array('link_id' => $link['id']);
}

/**
 * Publish photo action configuration form.
 */
function fbapp_rules_publish_photo_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '', 'source' => '', 'message' => '', 'no_story' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application posting the photo.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target feed to post on. Due to a bug in Facebook\'s API, in some cases (like when posting to a group) the id may not be correctly returned.'),
    '#required' => TRUE,
  );
  $form['settings']['source'] = array(
    '#type' => 'textfield',
    '#title' => t('Image URL'),
    '#default_value' => $settings['source'],
    '#description' => t('Absolute physical path to the image file to post, starting with leading "/".'),
    '#required' => TRUE,
  );
  $form['settings']['message'] = array(
    '#type' => 'textfield',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The photo caption.'),
    '#required' => TRUE,
  );
  $form['settings']['no_story'] = array(
    '#type' => 'checkbox',
    '#title' => t('Don\'t publish to user\'s wall'),
    '#default_value' => $settings['no_story'],
    '#description' => t('Check this option to inhibit publishing a story on the user\'s wall. If checked, the user will be notified that the application has posted a photo on his behalf.'),
  );
}

/**
 * Publish photo to Facebook.
 */
function fbapp_rules_publish_photo($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
    'source' => '@'. realpath('.'. base_path()) . $settings['source'],
    'message' => $settings['message'],
    // Used to get the photo ID in return.
    'fields' => 'id',
  );
  if ($settings['no_story'] == 1) {
    $parameters['no_story'] = 1;
  }
  $photo = fbapp_graph_request("{$target}/photos", $parameters, 'POST');
  // Return the created photo ID.
  return array('photo_id' => $photo['id']);
}

/**
 * Comment object configuration form.
 */
function fbapp_rules_comment_object_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '', 'message' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application posting the comment.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target object to comment on.'),
    '#required' => TRUE,
  );
  $form['settings']['message'] = array(
    '#type' => 'textfield',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
    '#description' => t('The comment text.'),
    '#required' => TRUE,
  );
}

/**
 * Comment object on Facebook.
 */
function fbapp_rules_comment_object($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
    'message' => $settings['message'],
    // Used to get the comment ID in return.
    'fields' => 'id',
  );
  $comment = fbapp_graph_request("{$target}/comments", $parameters, 'POST');
  // Return the created comment ID.
  return array('comment_id' => $comment['id']);
}

/**
 * Like object configuration form.
 */
function fbapp_rules_like_object_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application \'liking\' the object.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target object to like.'),
    '#required' => TRUE,
  );
}

/**
 * Like object on Facebook.
 */
function fbapp_rules_like_object($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
  );
  fbapp_graph_request("{$target}/likes", $parameters, 'POST');
}

/**
 * Unlike object configuration form.
 */
function fbapp_rules_unlike_object_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application \'unliking\' the object.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target object to unlike.'),
    '#required' => TRUE,
  );
}

/**
 * Unlike object on Facebook.
 */
function fbapp_rules_unlike_object($settings) {
  $target = $settings['target'];
  $parameters = array(
    'access_token' => $settings['access_token'],
  );
  fbapp_graph_request("{$target}/likes", $parameters, 'DELETE');
}

/**
 * Delete object action configuration form.
 */
function fbapp_rules_delete_object_form($settings, &$form) {
  $settings += array('access_token' => '', 'target' => '');

  $form['settings']['access_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Access token'),
    '#default_value' => $settings['access_token'],
    '#description' => t('The access token of the user or application deleting the object.'),
    '#required' => TRUE,
  );
  $form['settings']['target'] = array(
    '#type' => 'textfield',
    '#title' => t('Target ID'),
    '#default_value' => $settings['target'],
    '#description' => t('Facebook ID of the target object to delete.'),
    '#required' => TRUE,
  );
}

/**
 * Delete object from Facebook.
 */
function fbapp_rules_delete_object($settings) {
  $target = $settings['target'];
  if ($target == '') {
    return;
  }
  $parameters = array(
    'access_token' => $settings['access_token'],
  );
  fbapp_graph_request("{$target}", $parameters, 'DELETE');
}

